(* $Id: oOChannel.ml,v 1.3 2004/01/17 20:57:55 yori Exp $ *)
(* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)

class type ['a] obj_input_channel =
  object
    method get : 'a
    method close : unit
  end

class type ['a] obj_output_channel =
  object
    method put : 'a -> unit
    method flush : unit
    method close : unit
  end

class ['a] channel_of_stream s  =
  object
    val s = s
    method get :'a = 
      try Stream.next s with Stream.Failure -> raise End_of_file
    method close = ()
  end

let stream_of_channel inchan =
  Stream.from (fun _ ->
    try Some (inchan#get) with End_of_file -> None)

class ['a] queueing q =
  object
    method put (e : 'a) = Queue.add e q
    method flush = ()
    method close = ()
  end

class ['a, 'b] filter_in 
    (filter : 'b obj_output_channel -> 'a obj_output_channel)
    (c_in : 'a obj_input_channel)
    =
  let q = Queue.create () in
  let c_out = filter (new queueing q) in
  object (self)
    val mutable c_out_closed = false
    method get : 'b =
      try Queue.take q with Queue.Empty ->
	if c_out_closed then raise End_of_file else
	((try c_out#put (c_in#get) with End_of_file ->
	  c_out#close;
	  c_out_closed <- true);
	 self#get;)

    method close = 
      c_in#close;
      c_out#close;
      Queue.clear q;
  end

class ['a] unget c =
  let st = Stack.create () in
  object
    method get : 'a =
      if Stack.is_empty st then c#get else
      Stack.pop st
    method close : unit = c#close
    method unget e = Stack.push e st
  end

class type char_input_channel =
  object
    inherit [char] obj_input_channel
    method input : string -> int -> int -> int
    method really_input : string -> int -> int -> unit
  end

class char_input_channel_of (oc : char obj_input_channel) =
  object (self)
    method get = oc#get
    method close = oc#close
    method input b p len =
      for i = p to p+len -1 do b.[i] <- oc#get done;
      len
    method really_input b p len = 
      ignore(self#input b p len)
  end

class type char_output_channel =
  object
    inherit [char] obj_output_channel
    method output : string -> int -> int -> unit
  end

class char_output_channel_of (oc : char obj_output_channel) =
  object
    method put = oc#put
    method flush = oc#flush
    method close = oc#close
    method output b p len =
      for i = p to p+len-1 do oc#put b.[i] done
  end

class of_in_channel p_in =
  object
    method get = input_char p_in
    method close = close_in p_in
    method input b p len = input p_in b p len
    method really_input b p len = really_input p_in b p len
  end

class of_out_channel p_out =
  object
    method put = output_char p_out
    method close = close_out p_out
    method output = output p_out
    method flush = flush p_out
  end
