#include <locale.h>
#include <langinfo.h>
#include <wchar.h>
#include <wctype.h>
#include <limits.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/fail.h>

static int translate_category (value category){
  CAMLparam1 (category);

  switch (Long_val(category)){
  case 0: return LC_ALL;
  case 1: return LC_COLLATE;
  case 2: return LC_CTYPE;
  case 3: return LC_MONETARY;
  case 4: return LC_TIME;
  case 5: return LC_MESSAGES;
  default:
    invalid_argument ("Unknown Locale Category");
  }
}

value camomile_setlocale(value category, value locale){
  CAMLparam2 (category, locale);
  int c_category;

  c_category = translate_category (category);
  setlocale (c_category, String_val (locale));
  CAMLreturn (Val_unit);
}

value camomile_locale(value category){
  CAMLparam1 (category);
  int c_category;
  char *locale;

  c_category = translate_category (category);  
  locale = setlocale (c_category, NULL);
  CAMLreturn (copy_string (locale));
}

value camomile_encname(value unit){
  CAMLparam1 (unit);
  char *s;
  s = nl_langinfo (CODESET);
  if (s == NULL) s = "INVARIANT";
  CAMLreturn (copy_string (s));
}

value camomile_new_mbstate (value unit){
  CAMLparam1 (unit);
  mbstate_t *state;

  state = (mbstate_t *)malloc(sizeof (mbstate_t));
  memset(state, '\0', sizeof (mbstate_t));
  CAMLreturn ((value)state);
}

value camomile_delete_mbstate (value state){
  CAMLparam1 (state);

  free((mbstate_t *)state);

  CAMLreturn (Val_unit);
}

value camomile_mb_len_max (value unit){
  CAMLparam1 (unit);

  CAMLreturn (Val_int(MB_LEN_MAX));
}

value camomile_mbrtowc (value string, value len, value wbuf, value state){
  CAMLparam4 (string, len, wbuf, state);
  size_t n;
  wchar_t w = 0;

  n = mbrtowc (&w, 
	       String_val(string), 
	       Long_val(len),
	       (mbstate_t *)state);
  Store_field(wbuf, 0, Val_int(w));

  CAMLreturn (Val_int(n));
}

value camomile_wcrtomb (value string, value wchar, value state){
  CAMLparam3 (string, wchar, state);
  size_t n;

  n = wcrtomb (String_val(string), 
	       Long_val(wchar), 
	       (mbstate_t *)state);
  CAMLreturn (Val_int(n));
}

value camomile_iswprint (value wchar){
  CAMLparam1 (wchar);
  wchar_t w = Long_val (wchar);
  int b = 0;

  b = iswprint(w) ? 1 : 0;

  CAMLreturn (Val_int(b));
}
