(* $Id: rawByte.ml,v 1.1 2004/04/06 22:12:40 yori Exp $ *)
(* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)

module type PervasivesType = sig
  val open_out : string -> out_channel
  val open_out_gen : 
      open_flag list -> int -> string -> out_channel
  val flush : out_channel -> unit
  val output_char : out_channel -> char -> unit
  val output_string : out_channel -> string -> unit
  val close_out : out_channel -> unit

  val open_in : string -> in_channel
  val open_in_gen : open_flag list -> int -> string -> in_channel
  val input_char : in_channel -> char
  val input_line : in_channel -> string

  val stdin : in_channel
  val stdout : out_channel
  val stderr : out_channel

  val print_char : char -> unit
  val print_string : string -> unit
  val print_endline : string -> unit
  val print_newline : unit -> unit
      
  val prerr_char : char -> unit
  val prerr_string : string -> unit
  val prerr_endline :  string -> unit
  val prerr_newline : unit -> unit

  val read_line : unit -> string

  val (^) : string -> string -> string
end
module type CharType = sig
  external code : char -> int = "%identity"
  val chr : int -> char
  val escaped : char -> string
  val lowercase : char -> char
  val uppercase : char -> char
  type t = char
  val compare: t -> t -> int
  external unsafe_chr : int -> char = "%identity"
end
module type StringType = sig
  type t = string
  external length : string -> int = "%string_length"
  external get : string -> int -> char = "%string_safe_get"
  external set : string -> int -> char -> unit = "%string_safe_set"
  external create : int -> string = "create_string"
  val make : int -> char -> string
  val copy : string -> string
  val sub : string -> int -> int -> string
  val fill : string -> int -> int -> char -> unit
  val blit : string -> int -> string -> int -> int -> unit
  val concat : string -> string list -> string
  val iter : (char -> unit) -> string -> unit
  val escaped : string -> string
  val index : string -> char -> int
  val rindex : string -> char -> int
  val index_from : string -> int -> char -> int
  val rindex_from : string -> int -> char -> int
  val contains : string -> char -> bool
  val contains_from : string -> int -> char -> bool
  val rcontains_from : string -> int -> char -> bool
  val uppercase : string -> string
  val lowercase : string -> string
  val capitalize : string -> string
  val uncapitalize : string -> string
  val compare: t -> t -> int
  external unsafe_get : string -> int -> char = "%string_unsafe_get"
  external unsafe_set : string -> int -> char -> unit = "%string_unsafe_set"
  external unsafe_blit :
      string -> int -> string -> int -> int -> unit = "blit_string" "noalloc"
  external unsafe_fill :
	  string -> int -> int -> char -> unit = "fill_string" "noalloc"
end
module type BufferType = sig
  type t
  val create : int -> t
  val contents : t -> string
  val length : t -> int
  val clear : t -> unit
  val reset : t -> unit
  val add_char : t -> char -> unit
  val add_string : t -> string -> unit
  val add_substring : t -> string -> int -> int -> unit
  val add_substitute : t -> (string -> string) -> string -> unit
  val add_buffer : t -> t -> unit
  val add_channel : t -> in_channel -> int -> unit
  val output_buffer : out_channel -> t -> unit
end
include Pervasives
module Char = Char
module String = String
module Buffer = Buffer
