(* $Id: uTF8Byte.ml,v 1.1 2004/04/06 22:12:40 yori Exp $ *)
(* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)

open UPervasives

module UTF8NF = UNF.Make (UTF8)
module UTF8CaseMap = CaseMap.Make (UTF8)
module UTF8Col = UCol.Make (UTF8)

module NormalizationOff = struct
  module Pervasives : UTF8BytePervasives.Type = struct
    include UTF8Unicode
    let output_char uc c = uc#output_char c
    let output_string uc s = uc#output s 0 (String.length s)
    let close_out uc = uc#close

    let input_char uc = uc#input_char

    let print_char u = output_char stdout u
    let print_string s = output_string stdout s

    let prerr_char u = output_char stderr u
    let prerr_string s = output_string stderr s
  end
  include Pervasives

  module Char = RawByte.Char
  module String = struct
    include String

    let nfd = UTF8NF.nfd
    let nfkd = UTF8NF.nfkd
    let nfc = UTF8NF.nfc
    let nfkc = UTF8NF.nfkc

    let uppercase ?(locale=Locale.current_locale Locale.LC_CTYPE) s =
      UTF8CaseMap.uppercase ~locale s

    let lowercase ?(locale=Locale.current_locale Locale.LC_CTYPE) s =
      UTF8CaseMap.lowercase ~locale s

    let titlecase ?(locale=Locale.current_locale Locale.LC_CTYPE) s =
      UTF8CaseMap.titlecase ~locale s

    let capitalize ?(locale=Locale.current_locale Locale.LC_CTYPE) s =
      if length s <= 0 then s else
      let u = UTF8.look s 0 in
      let u = UTF8Unicode.Char.titlecase ~locale u in
      let b = UTF8.Buf.create 0 in
      UTF8.Buf.add_char b u;
      Buffer.add_substring b s (UTF8.next s 0) (length s - 1);
      Buffer.contents b

    let uncapitalize = lowercase

    let compare = UTF8.compare

    let compare_col
	?(locale=Locale.current_locale Locale.LC_COLLATE)
	?(strength=0)
	?(variable=`Shifted)
	(s1 : string) (s2 : string) =
      let prec =
	if strength = 1 then Some `Primary else
	if strength = 2 then Some `Secondary else
	if strength = 3 then Some `Tertiary else
	if strength = 4 then Some `Quaternary else None in
      UTF8Col.compare ~locale ?prec ~variable s1 s2

    let search_col
	?(locale=Locale.current_locale Locale.LC_COLLATE)
	?(strength=0)
	?variable
	(s1 : string) (s2 : string) i =
      let prec =
	if strength = 1 then Some `Primary else
	if strength = 2 then Some `Secondary else
	if strength = 3 then Some `Tertiary else
	if strength = 4 then Some `Quaternary else None in
      UTF8Col.search ~locale ?prec ?variable s1 s2 i
  end
  module Buffer = RawByte.Buffer
end

module NormalizationNFC = struct
  include NormalizationOff.Pervasives
  let input_line uc = UTF8NF.nfc (input_line uc)
  let read_line () = input_line stdin
  let (^) = UTF8NF.nfc_append
      
  module Char = Char
  module String = struct
    include NormalizationOff.String
    let concat pad sts =
      UTF8NF.nfc (String.concat pad sts)
  end
  module Buffer = struct
    include NormalizationOff.Buffer
    let contents b : string = UTF8NF.nfc (contents b)
  end
end
    
include NormalizationOff

